package com.astrum.io;

import java.io.IOException;
import java.util.concurrent.locks.ReentrantLock;

public abstract class IONative {

	static boolean isLoaded = false;
	static ReentrantLock libraryLock = new ReentrantLock();
	static {
		try {
			libraryLock.lock();
			if (!isLoaded) {
				System.loadLibrary("IOSDK");
				isLoaded = true;
			}
		} catch (Exception e) {
			IOLog.logE(e);
		} finally {
			libraryLock.unlock();
		}
	}
	protected long nativeId;
	protected boolean opened;
	private ReentrantLock lock = new ReentrantLock();

	public boolean open() throws IOException {
		try {
			lock.lock();
			if (opened) {
				return true;
			}
			opened = true;
			return (opened = IOOpen());
		} finally {
			lock.unlock();
		}
	}

	public boolean close() throws IOException {
		try {
			lock.lock();
			if (!opened) {
				return true;
			}
			opened = false;
			if (IOClose()) {
				opened = false;
				return true;
			} else {
				opened = true;
				return false;
			}
		} finally {
			lock.unlock();
		}

	}

	public boolean isOpened() {
		return opened;
	}

	protected abstract boolean IOOpen() throws IOException;

	protected abstract boolean IOClose() throws IOException;
}
