package com.astrum.io;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

public abstract class IOStream extends IONative {
	public enum FlushMode {
		IN, OUT, IN_OUT
	}

	protected byte[] buffer = new byte[2048 * 10];

	protected abstract int IOAvailable() throws IOException;

	protected abstract int IOFlush(FlushMode mode) throws IOException;

	protected abstract int IORead(int timeOut) throws IOException;

	protected abstract int IORead(byte[] buffer, int offset, int size, int timeOut) throws IOException;

	protected abstract int IOWrite(byte[] buffer, int offset, int size) throws IOException;

	public final int available() {
		int result = -1;
		try {
			result = IOAvailable();
		} catch (IOException e) {

		}
		return result;
	}

	public final int flush(FlushMode mode) {
		int result = -1;
		try {
			result = IOFlush(mode);
			Thread.sleep(10);
		} catch (Exception ex) {
			IOLog.logE(ex);
		} finally {
		}
		return result;

	}

	public final int flush() {
		int result = -1;
		try {
			result = IOFlush(FlushMode.IN_OUT);
			Thread.sleep(10);
		} catch (Exception ex) {

		} finally {
		}
		return result;

	}

	public final int write(String data) {
		try {
			byte[] buffer = data.getBytes("utf8");
			return write(buffer, 0, buffer.length);
		} catch (UnsupportedEncodingException e) {
			return -1;
		}
	}

	public final int write(Byte[] buffer, int offset, int size) {
		byte[] tmp = new byte[buffer.length];
		for (int i = 0; i < buffer.length; i++) {
			tmp[i] = buffer[i].byteValue();
		}
		return write(tmp, offset, size);
	}

	public final int write(byte[] buffer, int offset, int size) {
		int result = -1;
		try {
			if (isOpened()) {
				result = IOWrite(buffer, offset, size);
			}

		} catch (Exception ex) {
			IOLog.logE(ex);
		} finally {

		}
		return result;
	}

	public final int read() {
		return read(-1);
	}

	public final int read(int timeOut) {
		int result = -1;
		try {
			if (isOpened()) {
				result = IORead(timeOut);
			}
		} catch (Exception ex) {

		} finally {

		}
		return result;

	}

	public final int read(byte[] buffer, int offset, int size) {
		return read(buffer, offset, size, -1);
	}

	public final int read(byte[] buffer, int offset, int size, int timeOut) {
		int result = -1;
		try {
			if (isOpened()) {
				result = IORead(buffer, offset, size, timeOut);
			}
		} catch (Exception ex) {

		} finally {
		}
		return result;
	}

	public final List<String> readLines() {
		return readLines(-1);
	}

	public final List<String> readLines(int timeOut) {
		List<String> resultList = new ArrayList<>();
		String result = readLine(timeOut);
		if (result != null) {
			String[] lines = result.split("\\n");
			for (String l : lines) {
				resultList.add(l.trim());
			}
		}
		return resultList;
	}

	public final String readLine(int timeOut) {

		int index = read(buffer, 0, buffer.length, timeOut);
		if (index > 0) {
			try {
				return new String(buffer, 0, index, "utf8");
			} catch (Exception e) {

			}

		}
		return null;
	}
}
