package com.astrum.sample;

import java.io.IOException;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.graphics.Color;
import android.os.Bundle;
import android.support.v4.app.NavUtils;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.RelativeLayout;
import android.widget.Spinner;
import android.widget.TabHost;
import android.widget.TextView;
import android.widget.Toast;

import com.astrum.io.IO485;
import com.astrum.io.IOLog;
import com.astrum.io.IOStream;
import com.astrum.io.IOUART;
import com.astrum.io.IOUART.Ports;
import com.astrum.sdk.R;

public class IO485Activity extends Activity {
    static final String LOG_TAG = "IO485";
    IOStream uart = null;
    EditText txtHexReceive;
    EditText txtAsciReceive;
    TextView txtSendText;
    Button btnSend;
    Button btnClear;
    TabHost tabHost;
    RadioButton rdoHex;
    RadioButton rdoAsci;
    CheckBox chkAutoClear;
    SettingDialog settingDialog;
    Thread receiveThread;
    IOLog.IOLogEventHandler logEvents = new IOLog.IOLogEventHandler() {
        @Override
        public void onLog(String log) {
            Log.i(LOG_TAG, log);
        }

        @Override
        public void onLogE(String log) {
            Log.e(LOG_TAG, log);
        }

        @Override
        public void onLogSystemError(String log, String Detail) {
            Log.e(LOG_TAG, Detail);
        }

        @Override
        public void onLogI(String log) {
            Log.i(LOG_TAG, log);
        }

        @Override
        public void onLogD(String log) {
            Log.d(LOG_TAG, log);
        }

        @Override
        public void onLogW(String log) {
            Log.w(LOG_TAG, log);
        }
    };

    @Override
    public void onCreate(Bundle savedInstanceState) {
        IOLog.addLogListener(logEvents);
        super.onCreate(savedInstanceState);
        settingDialog = new SettingDialog(this);
        setContentView(R.layout.io485_activity);
        txtHexReceive = (EditText) findViewById(R.id.txtHexReceive);
        txtAsciReceive = (EditText) findViewById(R.id.txtAsciReceive);
        txtSendText = (TextView) findViewById(R.id.txtSendText);
        btnSend = (Button) findViewById(R.id.btnSend);
        btnClear = (Button) findViewById(R.id.btnClear);
        tabHost = (TabHost) findViewById(R.id.tabHost);
        rdoHex = (RadioButton) findViewById(R.id.rdoHex);
        rdoAsci = (RadioButton) findViewById(R.id.rdoAsci);
        chkAutoClear = (CheckBox) findViewById(R.id.chkAutoClear);
        txtSendText.requestFocus();
        rdoAsci.setChecked(true);
        tabHost.setup();
        txtSendText.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View view, boolean b) {
                if (!b) {
                    InputMethodManager imm = (InputMethodManager) getSystemService(Activity.INPUT_METHOD_SERVICE);
                    imm.toggleSoftInput(InputMethodManager.HIDE_IMPLICIT_ONLY, 0);
                }
            }
        });
        TabHost.TabSpec tab1 = tabHost.newTabSpec("Hex");
        TabHost.TabSpec tab2 = tabHost.newTabSpec("Asci");
        tab1.setContent(R.id.tab1);
        tab1.setIndicator("Hex Receive");
        tab2.setContent(R.id.tab2);
        tab2.setIndicator("Asci Receive");
        tabHost.addTab(tab1);
        tabHost.addTab(tab2);
        txtSendText.setText("303132333435363738");
        btnSend.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (uart == null) {
                    return;
                }
                txtSendText.clearFocus();
                btnSend.requestFocus();
                if (rdoHex.isChecked()) {
                    try {
                        String text = txtSendText.getText().toString().trim();
                        byte[] data = new byte[text.length() / 2];
                        for (int i = 0; i < text.length(); i += 2) {
                            String value = text.substring(i, i + 2);
                            data[i / 2] = (byte) Integer.parseInt(value, 16);
                        }
                        uart.write(data, 0, data.length);
                        txtSendText.setBackgroundColor(Color.TRANSPARENT);
                        if (chkAutoClear.isChecked()) {
                            txtSendText.setText("");
                        }
                    } catch (Exception e) {
                        txtSendText.setBackgroundColor(Color.RED);
                        Toast.makeText(IO485Activity.this, e.toString(), Toast.LENGTH_SHORT).show();
                    }
                } else {
                    try {
                        String text = txtSendText.getText().toString();
                        text = text.replaceAll("\n", "\r\n");
                        byte[] data = text.getBytes("UTF-8");
                        uart.write(data, 0, data.length);
                        txtSendText.setBackgroundColor(Color.TRANSPARENT);
                        if (chkAutoClear.isChecked()) {
                            txtSendText.setText("");
                        }
                    } catch (Exception e) {
                        txtSendText.setBackgroundColor(Color.RED);
                        Toast.makeText(IO485Activity.this, e.toString(), Toast.LENGTH_SHORT).show();
                    }
                }
            }
        });
        btnClear.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                txtHexReceive.setText("");
                txtAsciReceive.setText("");
            }
        });
        settingDialog.show();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater menuInflater = getMenuInflater();
        menuInflater.inflate(R.menu.menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.menu_settings:
                settingDialog.show();
                return true;
            case R.id.menu_exit:
                if (uart != null && uart.isOpened()) {
                    try {
                        uart.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
                android.os.Process.killProcess(android.os.Process.myPid());
                return true;
            case android.R.id.home:
                NavUtils.navigateUpFromSameTask(this);
                return true;

            default:
                return super.onOptionsItemSelected(item);
        }
    }

    class ReceiveThread extends Thread {
        @Override
        public void run() {
            byte[] buffer = new byte[1024];
            IOLog.logD("Opened Port");
            while (uart.isOpened()) {
                int result = uart.read(buffer, 0, buffer.length);
                if (result > 0) {
                    final String asciData = new String(buffer, 0, result);
                    String hex = "";
                    for (int i = 0; i < result; i++) {
                        hex += String.format("%02X ", buffer[i]);
                    }
                    final String finalHex = hex;
                    txtHexReceive.post(new Runnable() {
                        @Override
                        public void run() {
                            txtHexReceive.setText(txtHexReceive.getText().toString() + finalHex);
                            txtAsciReceive.setText(txtAsciReceive.getText().toString() + asciData);
                        }
                    });
                }
            }
            IOLog.logD("Closed Port");
        }
    }

    class SettingDialog {
        Spinner cboPort;
        Spinner cboBaudrate;
        Spinner cboParity;
        Spinner cboDataBit;
        Spinner cboStopBit;
        Dialog dialog;

        public SettingDialog(Context c) {
            dialog = new Dialog(IO485Activity.this);
            dialog.getWindow().setLayout(RelativeLayout.LayoutParams.MATCH_PARENT, RelativeLayout.LayoutParams.MATCH_PARENT);
            dialog.setContentView(R.layout.setting);
            dialog.setCancelable(false);
            dialog.setTitle("Settings");
            cboBaudrate = (Spinner) dialog.findViewById(R.id.cboBaudrate);
            cboDataBit = (Spinner) dialog.findViewById(R.id.cboDataBit);
            cboPort = (Spinner) dialog.findViewById(R.id.cboSerialPort);
            cboStopBit = (Spinner) dialog.findViewById(R.id.cboStopBit);
            cboParity = (Spinner) dialog.findViewById(R.id.cboParity);
            cboPort.setSelection(0);
            cboBaudrate.setSelection(4);
            cboDataBit.setSelection(0);
            cboStopBit.setSelection(1);
            cboParity.setSelection(0);
            Button btnCancel = (Button) dialog.findViewById(R.id.btnCancel);
            Button btnOk = (Button) dialog.findViewById(R.id.btnOk);
            View.OnClickListener l = new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (v.getId() == R.id.btnOk) {
                        String port = cboPort.getSelectedItem().toString();
                        String baudRate = cboBaudrate.getSelectedItem().toString();
                        String sparity = cboParity.getSelectedItem().toString();
                        String sstopBit = cboStopBit.getSelectedItem().toString();
                        String sdataBit = cboDataBit.getSelectedItem().toString();
                        IOUART.Parity parity = IOUART.Parity.None;
                        if (sparity.equals("none")) {
                            parity = IOUART.Parity.None;
                        } else if (sparity.equals("Even")) {
                            parity = IOUART.Parity.Even;
                        } else if (sparity.equals("Mark")) {
                            parity = IOUART.Parity.Mark;
                        } else if (sparity.equals("Odd")) {
                            parity = IOUART.Parity.Odd;
                        } else if (sparity.equals("Space")) {
                            parity = IOUART.Parity.Space;
                        }
                        IOUART.StopBits stopBit = IOUART.StopBits.One;
                        if (sstopBit.equals("0")) {
                            stopBit = IOUART.StopBits.None;
                        } else if (sstopBit.equals("1")) {
                            stopBit = IOUART.StopBits.One;
                        } else if (sstopBit.equals("1.5")) {
                            stopBit = IOUART.StopBits.OnePointFive;
                        } else if (sstopBit.equals("2")) {
                            stopBit = IOUART.StopBits.Two;
                        }
                        if (uart != null && uart.isOpened()) {
                            try {
                                uart.close();
                            } catch (IOException e) {
                                e.printStackTrace();
                            }
                            if (receiveThread != null) {
                                try {
                                    receiveThread.join();
                                } catch (InterruptedException e) {
                                    e.printStackTrace();
                                }
                                receiveThread = null;
                            }
                        }
                        try {
                            uart = new IO485(Ports.index(port), Integer.parseInt(baudRate), parity, sdataBit.equals("8") ? 8 : 9, stopBit);
                            uart.open();
                            IO485Activity.this.getWindow().setTitle(IO485Activity.this.getResources().getText(R.string.activity_name) + " (Port:" + port + " Baudrate:" + baudRate + " Parity:" + parity + " Data Bit:" + sdataBit + " Stop Bit:" + stopBit + ")");
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                        receiveThread = new ReceiveThread();
                        receiveThread.start();
                    }
                    dialog.dismiss();
                }
            };
            btnCancel.setOnClickListener(l);
            btnOk.setOnClickListener(l);
        }

        public void show() {
            dialog.show();
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        getActionBar().setDisplayHomeAsUpEnabled(true);
    }

    @Override
    protected void onPause() {
        super.onPause();
        if (uart != null && uart.isOpened()) {
            try {
                uart.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}